//Player.c
#include <string.h> //memcmp
#include <stdio.h>	//sprintf
#include "SPI_SD.h"
#include "Player.h"
#include "I2S.h"

BYTE databuffer[512];
ULONG CachedSectorAddress;	//DataBuffer address

BYTE playbuffer[512];

//FAT information
BYTE FATType;			//0:FAT12, 1:FAT16
ULONG SectPerClust;		//Sectors in Cluster
ULONG RootDirEntrys; 	//Entrys in Root Sector
ULONG FATStartSect;		//Sector, FAT starts
ULONG DIRStartSect;		//Sector, Root Dir starts
ULONG DataStartSect;	//Sector, Data Area starts
FILEINFO WAVFiles[32];	//max 32 wave file can be played.

//Music Information
ULONG MusicNum;			//Playing Musics Number, from 1
ULONG TotalMusics;		//Total WAV File Num, from 1
ULONG CurrentClust;		//Playing Cluster
ULONG StartSectofClust;	//First Sector in current Playing Cluster
ULONG CurrentSectinClust;	//Playing Sector in Playing Cluster
ULONG SectorCounter;	//Counter for Sector in cluster, from 0,  < SectPerClust
ULONG RemainingSect;	//Sector remain in current music
ULONG PlayedSect;		//Sector Played in current music
						//These are 512bytes/128samples/2.9ms

//Basics of Operation
//Sector: 512Byte block, reading minimum unit from SD card.
//Cluster: 64 Sector = 32KB block in SD card.
//FAT sectors: Cluster address listing table. 256 index in one sector.
// 3 minutes of wave file size is about 31MB, it contains > 1000 cluster.

//reading one Sector needs 1.4ms.
//One Sector, 512Byte is consumed in 2.9ms.
//So reading cluster between playing can cause glitch.

//CBs will store Cluster chain listing.
// if Cluster chain is 0x0004 to 0x0024, 0x0045 to 0x0208
//	CBs[0].Start_Cluster = 0x0004
//	CBs[0].End_Cluster = 0x0024
//	CBs[1].Start_Cluster = 0x0045
//	CBs[1].End_Cluster = 0x0208

// Pros:So there are no FAT sector lookup when playing music.
// Cons:too much flagmented, chains > 128 file can not be played. maybe never happen by NORMAL operation
CLUSTERBLOCK CBs[128];
WORD CurrentCBIndex;

//DMA Flag, controlled by I2S.c
extern BYTE DMA_Flag_A;
//DMA Buffer, exists in I2S.c
extern WORD dci1TxBuffA[512] __attribute__((space(dma)));
extern WORD dci1TxBuffB[512] __attribute__((space(dma)));
//0x 00:00 buffer
char MinSec[10];

BYTE Player_Init(void)
{
	BYTE ret = 0;
	CachedSectorAddress = 0;
	
	memset(WAVFiles,0x00,sizeof(WAVFiles));

	//read FAT information, store into local variables	
	ret = read_FAT_info();
		
	if (ret != 0) return ret;
	
	//search wave files.
	ret = search_WAV(32);

	TotalMusics = ret;
	MusicNum = 1;
	//do simple sort, by file name
	SortMusic();
	//just initializing variables: set again at Player_GotoMusicHead
	CurrentClust = 0;
	CurrentSectinClust = 0;
	RemainingSect = 0;
	PlayedSect = 0;
		
	return ret;
}

void Player_NusicName(char * dest)
{
	memcpy(dest, WAVFiles[MusicNum - 1].fname, 8);	
}	
DWORD Player_FileSize(void)
{
	return WAVFiles[MusicNum -1].fsize;	
}	

UINT Player_MusicNum(void)
{
	return (UINT)MusicNum; 	
}	

//For starting info
void Player_TotalMinSec(char * dest)
{
	memset(MinSec,0x00,sizeof(MinSec));
	
	//return mm:ss as Total minutes:second
	UINT TotalSecond = 0;
	UINT Minutes = 0;
	UINT Seconds = 0;
	
	TotalSecond = ((WAVFiles[MusicNum -1].fsize)>>10) / 176;	
	Minutes = (UINT)(TotalSecond / 60);
	Seconds = TotalSecond % 60;
	
	sprintf(&MinSec[0],"%02u ", (UINT)(MusicNum & 0x00FF));
	sprintf(&MinSec[3],"%02u", Minutes);
	sprintf(&MinSec[5],":");
	sprintf(&MinSec[6],"%02u", Seconds);
	
	memcpy(dest, MinSec, 8);
}	

//Playing info, erapsed min:sec
void Player_MinSec(char * dest)
{
	memset(MinSec,0x00,sizeof(MinSec));
	
	//return mm:ss as played minutes:second
	UINT PlayedSecond = 0;
	UINT Minutes = 0;
	UINT Seconds = 0;
	
	PlayedSecond = (PlayedSect * 30) >> 10 ;	
	Minutes = (UINT)(PlayedSecond / 60);
	Seconds = PlayedSecond % 60;
	
	sprintf(&MinSec[0],"%02u ", (UINT)(MusicNum & 0x00FF));
	sprintf(&MinSec[3],"%02u", Minutes);
	sprintf(&MinSec[5],":");
	sprintf(&MinSec[6],"%02u", Seconds);
	
	memcpy(dest, MinSec, 8);
}	


void Player_GotoMusicHead(ULONG Music)
{
	WORD SearchingClust = 0;
	WORD NextClust = 0;
	
	//Go to Starting Cluster
	CurrentClust = WAVFiles[Music - 1].org_clust;
	//Get First Sector Address in Cluster
	StartSectofClust = (CurrentClust -2) * SectPerClust + DataStartSect;
	//Current Sector is starting Sector in Cluster
	CurrentSectinClust = StartSectofClust;
	//SectorCounter is 0
	SectorCounter = 0;
	//All sector is remaining
	RemainingSect = ((WAVFiles[Music - 1].fsize) >> 9);
	//Nothing Played yet
	PlayedSect = 0;
	
	//Get Cluster Listing
	memset(CBs, 0x00, sizeof(CBs));
	CurrentCBIndex = 0;
	
	CBs[CurrentCBIndex].Start_Clust = WAVFiles[Music-1].org_clust;
	SearchingClust = CBs[CurrentCBIndex].Start_Clust;
	while(CurrentCBIndex < 128 - 1 )
	{
		NextClust = GetnextClusterByFAT(SearchingClust);
		if (NextClust == (SearchingClust + 1))
		{
			//continuous cluster.
			SearchingClust += 1;
		}
		else
		{
			//case, end of cluster chain
			if ((NextClust == 0xFFFF)||(NextClust == 0x0000))
			{
				CBs[CurrentCBIndex].End_Cluster = SearchingClust;
				break;
			}
			//jump to another location
			else
			{
				CBs[CurrentCBIndex].End_Cluster = SearchingClust;
				CBs[CurrentCBIndex+1].Start_Clust = NextClust;
				CBs[CurrentCBIndex+1].End_Cluster = NextClust;	//avoid missing link
				CurrentCBIndex++;
			}
		}
	}
	CurrentCBIndex = 0;	//reset cluster-block index.
}	

void Player_Stop(void)
{
	//Stop SPDIF Stream
	initI2sBuff();	
	i2s_stop();
	//Player goes to first music, first Cluster, first sector
	MusicNum = 1;
	Player_GotoMusicHead(MusicNum);
}
void Player_Start(void)
{
	dma0_start();	//restart DMA
	//start SPDIF Stream
	i2s_start();
}	

void Player_Pause(void)
{
	//SPDIF Stream is set to all zero. keep stream.
	initI2sBuff();
}	

void Player_FF(void)
{
	//go to next music
	MusicNum += 1;
	if (MusicNum > TotalMusics)
	{
		MusicNum = TotalMusics;
	}
	//goto head of music
	Player_GotoMusicHead(MusicNum);
}

void Player_Prev(void)
{
	//go to previous music
	MusicNum -= 1;
	if (MusicNum < 1) 
	{
		MusicNum = 1;
	}
	//goto head of music
	Player_GotoMusicHead(MusicNum);
}	

BYTE Player_Play(void)
{
	ULONG TailFillerSize = 0;
	int i = 0;
	BYTE ret = 0;
	BYTE ToContinue = 1;	//initially, continue mode
	
	ret = SD_Read_Single_Block(CurrentSectinClust * 512, playbuffer);

	//Modify Head and Tail sector
	if (PlayedSect == 0)	//This is first sector, includes headder
	{
		//fill WAVE headder with 0x00
		memset(playbuffer, 0x00, 44);
	}
	if (RemainingSect < 1)	//now this is last sector.
	{
		//Fill tail of sector with 0x00
		TailFillerSize = (WAVFiles[MusicNum - 1].WaveSize + 44) % 512;
		TailFillerSize &= 0x000001FF;	//limit 512 to avoid miss-fill. just in case..
		memset(playbuffer + (512 - TailFillerSize), 0x00, (512 - TailFillerSize));
		ToContinue = 0;
	}

	//Dump data to DMA Memory
	if (DMACS1 == 0x0001)	//DMA0STB is working. fill Page A.
	{
		for (i = 0; i < 256; i++)
		{
	        dci1TxBuffA[2*i]=(WORD)playbuffer[2*i] + (((WORD)playbuffer[2*i+1]) << 8) ; 	//data
	        dci1TxBuffA[2*i+1]=0x0000;	//Padding
		}
	}
	if (DMACS1 == 0x0000)	//DMA0STA is working. fill Page B.
	{
		for (i = 0; i < 256; i++)
		{
		    dci1TxBuffB[2*i]=(WORD)playbuffer[2*i] + (((WORD)playbuffer[2*i+1]) << 8) ; 	//data
		    dci1TxBuffB[2*i+1]=0x0000;	//Padding
		}
	}
	
	if (ToContinue == 1)
	{	
		//move to next sector.
		SectorCounter += 1;			// counting 00 to 63 (when 64 sectors in cluster)
		PlayedSect += 1;			// Total Played Sector
		RemainingSect -= 1;			// Remaining Sector
		//Check Sector is in Cluster
		if (SectorCounter > (SectPerClust - 1) ) // when 63+1 > 64-1
		{
			//go to next cluster.
			CurrentClust = GetnextCluster(CurrentClust);
			if (CurrentClust != 0)
			{
				StartSectofClust = (CurrentClust -2) * SectPerClust + DataStartSect;
				CurrentSectinClust = StartSectofClust;
				SectorCounter = 0;
			}
			else
			{
				ToContinue = 0; //end of cluster
			}
		}
		else
		//just step to next sector. it will be used next time.
		{
			CurrentSectinClust += 1;
		}
	}
	return ToContinue;
}	

ULONG GetnextCluster(ULONG CurrentClust)
{
	ULONG NextClust = 0;

	if (CurrentClust < CBs[CurrentCBIndex].End_Cluster)
	{
		NextClust = CurrentClust + 1;
	}
	else
	{
		CurrentCBIndex++;
		NextClust = CBs[CurrentCBIndex].Start_Clust;
	}
	return NextClust;
}

WORD GetnextClusterByFAT(WORD CurrentClust)
{
	ULONG SectAddr = 0;
	BYTE ret = 0;
	WORD NextClust = 0;
	WORD indexofClust = 0;

	//there are 256 cluster in one FAT table.
	SectAddr = (FATStartSect + (CurrentClust >> 8)) * 512;
	
	if (CachedSectorAddress == SectAddr)
	{
		//databuffer already containes same information.
	}
	else
	{
		//read FAT Sector and,
		ret = SD_Read_Single_Block(SectAddr, databuffer);
		//remember address
		CachedSectorAddress = SectAddr;
	}
	
	indexofClust = CurrentClust & 0x00FF; //tail 8bit = 0 to 256
	NextClust = (WORD)databuffer[indexofClust*2] + (((WORD)databuffer[indexofClust*2 + 1])<<8); 
	
	if (NextClust >= 0xFFF8) NextClust = 0xFFFF;	//end of cluster chain
		
	return NextClust;
}

BYTE read_FAT_info(void)
{
	ULONG bpb;			// Bios Parameter Block Sector Number.
	ULONG sectlen;		// Byte Length per Sector, should be 512.
	ULONG FATSects;		// FAT system starting Sector number.
	BYTE ret;			// return value from other function
	ULONG ROOTSects;	// How many sectors in Root Directory Entry structures
	
	bpb = 0;
	sectlen = 0;
	FATSects = 0;
	ret = 0;
	ROOTSects = 0;
	
	//Read MBR
	ret = SD_Read_Single_Block(0, databuffer);
	CachedSectorAddress = 0;
	
	if (ret == 0)
	{
		//read Bios Parameter Block sector number
		bpb = (ULONG)databuffer[MBR_START+MBR_BPP1] + (((ULONG)databuffer[MBR_START+MBR_BPP1+1])<<8);
	}
	else
	{ return ret; }
		
	FATType = 0;
	if (databuffer[MBR_START + 4] == 0x06) FATType = 1;
	
	// Read Bios Parameter Block
	ret = SD_Read_Single_Block(bpb * 512, databuffer);
	CachedSectorAddress = bpb * 512;
	
	if (ret == 0)
	{
		//read Sector Length
		sectlen = (ULONG)databuffer[BPB_SECTLEN] + ((ULONG)databuffer[BPB_SECTLEN+1]<<8);
	}
	else
	{ return ret; }

	if (sectlen != 512) return 2;	//not 512 Bytes per sector. error.
	
	//read other information in Bios Parameter Block 
	SectPerClust = (ULONG)databuffer[BPB_SECTPERCLUST];
		//Cluster is 4KByte when you see harddisk, unit of smallest file area.
	FATStartSect = bpb + (ULONG)databuffer[BPB_RESVDSECT]+(((ULONG)databuffer[BPB_RESVDSECT+1])<<8);
		//FAT Sector
	FATSects = (ULONG)databuffer[BPB_FATSIZE]+(((ULONG)databuffer[BPB_FATSIZE+1])<<8);
		//How many FAT sector exist
	if (FATSects ==0) return 2; //not FAT16
	
	DIRStartSect = FATStartSect + (ULONG)databuffer[BPB_NUMOFFATS] * FATSects;
		// Root directory startinc sector
	RootDirEntrys = (ULONG)databuffer[BPB_ROOTENT]+(((ULONG)databuffer[BPB_ROOTENT+1])<<8);
		// normally this is fixed, 512 (files, dirs in root directory can be exist)
	ROOTSects = RootDirEntrys / (512 / 32);
		// it should be 32.
	DataStartSect = DIRStartSect + ROOTSects;
		// User data starting sector number

	return 0;	
	
}	

BYTE search_WAV(UINT MaxCount)	// Upper Limit to search WAV files
{
	ULONG EntrySector=0; 	//Sector Number for Directory Entry
	ULONG sectaddr=0;		//Actual Sector Address
	ULONG MusicFound=0;	//How many WAV found
	ULONG EntryNum=0;		//There are 16 structure(32bytes) in one sector(512bytes)
	BYTE ret=0;			//return value from other function
	ULONG FirstClust=0;	//First Cluster number of found ***.WAV file
	ULONG FirstSect=0;	//First Sector number of found ***.WAV file
	

	for (EntrySector = 0 ; EntrySector < RootDirEntrys/(512 / 32); EntrySector++)
	{
		//DIRStartSect is calculated at read_FAT_info
		sectaddr = (DIRStartSect + EntrySector) * 512;
		//Read Sector Data
		ret = SD_Read_Single_Block(sectaddr, databuffer);
		CachedSectorAddress = sectaddr;
		//Search Directory Entry in Sector. There are 16 entry in one sector.
		for (EntryNum = 0; EntryNum < 16; EntryNum++)
		{
			//Directory Entry structure
			//0-10	File name, 8.3
			//11 	File attribute
			//12	reserved
			//13-19	time stamps
			//20-21	first cluster, upper 2 byte. always 0 for FAT16
			//22-25 time stamps
			//26-27	first cluster, lower 2 byte.
			//28-31	file size in ULONG
			
			//Check attribute. 0x10: directory flag, 0x80: volume flag
			if ( !(databuffer[EntryNum * 32 + 11] & 0x18))
			{
				//Check Cluster number: 
				FirstClust = (ULONG)databuffer[EntryNum * 32 + 26] + (((ULONG)databuffer[EntryNum * 32 + 27])<<8);
				if ( ( FirstClust > 0 ) &&			 				//start cluster not = 0,
				     ( databuffer[EntryNum * 32 + 0] != 0xE5) &&	// filename first byte is not deleted flag,
				     ( databuffer[EntryNum * 32 + 8]='W')&&
				     ( databuffer[EntryNum * 32 + 9]='A')&&
				     ( databuffer[EntryNum * 32 + 10]='V')			// extention is WAV
				   )
				{
					//Check Initial Sector for WAV format.44100/16/Stereo only.
					FirstSect = (FirstClust - 2) * SectPerClust + DataStartSect;
					ret = SD_Read_Single_Block(FirstSect * 512, playbuffer);
				    if ( ( memcmp( playbuffer, "RIFF",4) == 0) && 			// start with RIFF
					     ( memcmp( &playbuffer[8], "WAVEfmt ", 8) ==0 ) && 	// Format is WAVE
					     ( playbuffer[20] == 1 ) &&							// Linear PCM
					     ( playbuffer[22] == 2 ) &&							// Stereo
					     ( playbuffer[24] == 0x44 ) &&
					     ( playbuffer[25] == 0xAC ) &&						// 0xAC44 = 44100
					     ( playbuffer[32] == 4 ) &&							// 4 byte / sample
					     ( playbuffer[34] == 16 ))							// 16 bit
					   {
					     // this is wave file.
					     // Store File information.
					     WAVFiles[MusicFound].fsize = 	 (DWORD)databuffer[EntryNum * 32 + 28] +
					     					 			((DWORD)databuffer[EntryNum * 32 + 29]<<8) +
					     					 			((DWORD)databuffer[EntryNum * 32 + 30]<<16) +
					     					 			((DWORD)databuffer[EntryNum * 32 + 31]<<24);
					     WAVFiles[MusicFound].org_clust = FirstClust;
					     WAVFiles[MusicFound].WaveSize = (DWORD)playbuffer[40] +
					     					 			((DWORD)playbuffer[41]<<8) +
					     					 			((DWORD)playbuffer[42]<<16) +
					     					 			((DWORD)playbuffer[43]<<24);
					     
					     memcpy(WAVFiles[MusicFound].fname, (char *)&(databuffer[EntryNum * 32 + 0]), 8);
					     MusicFound++; // incremented after stored, so this value is [0 indexed] + 1 
					     MaxCount--;
	  	  				    }
				     if (MaxCount == 0) break;
				} // File, Cluster check IF
			} // not (directory or volume) check
		} // Entry in Sector LOOP
	} // Sector for Entry group LOOP
	return MusicFound;
}	

void SortMusic(void)
{
	//Simple Sort algorithm, sort FILINFO by File Name
	FILEINFO tempInfo;	// temp area to keep copy
	int i,j;
	// for 0 to Musics - 1, ex: if 1-10 musics found, i = 0 to 9. 
	for (i = 0; i < TotalMusics; i++)
	{
		// if 10 music found, j = 1 to 9.
		for (j = i + 1; j < TotalMusics; j++)
		{
			if ( ( strncmp(WAVFiles[j].fname, WAVFiles[i].fname, 8) ) < 0 )
			// if [j] filename smaller than [i] filename
			{
				//[j] and [i] exchanged. [i] -> temp, [j] -> [i], temp -> [i]
				memcpy((FILEINFO *)&tempInfo,   (FILEINFO *)&WAVFiles[i],sizeof(FILEINFO));
				memcpy((FILEINFO *)&WAVFiles[i],(FILEINFO *)&WAVFiles[j],sizeof(FILEINFO));
				memcpy((FILEINFO *)&WAVFiles[j],(FILEINFO *)&tempInfo,   sizeof(FILEINFO));
			}
		}
	}
}	



